//
//This sample program is provided AS IS and may be used, executed, copied and
//modified without royalty payment by customer (a) for its own instruction and
//study, (b) in order to develop applications designed to run with an IBM
//WebSphere product, either for customer's own internal use or for redistribution
//by customer, as part of such an application, in customer's own products. "
//
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2009
//All Rights Reserved * Licensed Materials - Property of IBM
//

import com.ibm.websphere.objectgrid.ClientClusterContext;
import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.ObjectGridManagerFactory;
import com.ibm.websphere.objectgrid.ObjectGridRuntimeException;
import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.Session;


/**
 * This simple ObjectGrid client that connects to a remote ObjectGrid
 * and allows CRUD operations from the command-line.<p>
 *
 * For details on how to run this program see the GETTINGSTARTED_README.txt
 * file or the ECLIPSE_README.txt included with this sample.
 */
public class Client {

    /**
     * Connect to a remote ObjectGrid
     * @param cep the catalog server end points in the form: <host>:<port>
     * @param gridName the name of the ObjectGrid to connect to that is managed by the Catalog Service
     * @return a client ObjectGrid connection.
     */
	static public ObjectGrid connectClient(String cep, String gridName)
	{
		try
		{
		    // Connect to the Catalog Server.  The security and client override XML are not specified.
			ClientClusterContext ccc = ObjectGridManagerFactory.getObjectGridManager().connect(cep, null, null);

			// Retrieve the ObjectGrid client connection and return it.
			ObjectGrid grid = ObjectGridManagerFactory.getObjectGridManager().getObjectGrid(ccc, gridName);
			return grid;
		}
		catch(Exception e)
		{
			throw new ObjectGridRuntimeException("Unable to connect to catalog server at endpoints:" + cep, e);
		}
	}
	/**
	 * The program main.  There are four arguments:<p>
	 * [0] The catalog server end points in the form <host>:<port>
	 * [1] The single-character ObjectGrid CRUD operation to perform:  i=insert, u=update, d=delete, g=get
	 * [2] The key.
	 * [3] The value.
	 *
	 * @param args the arguments as previously defined.
	 */
	public static void main(String[] args)
		throws Exception
	{
		if (args.length < 3){
			printUsage();
			return;
		}

		// Connect to a remote ObjectGrid
		ObjectGrid grid = connectClient(args[0], "Grid");

		// Get a session
		Session sess = grid.getSession();

		try {
            // Get the ObjectMap
            ObjectMap map1 = sess.getMap("Map1");

            // Invoke the CRUD operation using auto-commit transactions.
            if(args[1].equals("i"))
            {
            	map1.insert(args[2], args[3]);
            	System.out.println("SUCCESS: Inserted " + args[3] + " with key " + args[2]);
            }
            else if(args[1].equals("u"))
            {
            	map1.update(args[2], args[3]);
            	System.out.println("SUCCESS: Updated key " + args[2] + " with value " + args[3]);
            }
            else if(args[1].equals("d"))
            {
            	map1.remove(args[2]);
            	System.out.println("SUCCESS: Deleted value with key " + args[2]);
            }
            else if(args[1].equals("g"))
            {
            	String value = (String)map1.get(args[2]);
            	if(value != null)
            		System.out.println("Value is " + value);
            	else
            		System.out.println("Not found");
            }
            else
            {
            	System.out.println("Unknown operation: " + args[1]);
            	printUsage();
            }
        } finally {
            // Close the session (available since version 7.1.1)
            sess.close();
        }
	}

	private static void printUsage(){
	    System.out.println("This program executes simple CRUD operations on a map.");
	    System.out.println();
	    System.out.println("script usage: runclient.sh/runclient.bat <command: i, u, d, or g> <value1> [<value2>]");
	    System.out.println("java usage: java <jvm args> Client <catalog endpoint hostname:port> <command: i, u, d, or g> <value1> [<value2>]");
	    System.out.println();
	    System.out.println("\ti: insert value2 into grid with key value1");
	    System.out.println("\tu: update object keyed by value1 to value2");
	    System.out.println("\td: delete object keyed by value1");
	    System.out.println("\tg: retrieve and display object keyed by value1");
	}
}
